# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.helpers import destructiveTest, generate_random_name
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch
from tests.support.unit import TestCase, skipIf

# Import Salt Libs
import salt.utils.platform
import salt.utils.win_reg as win_reg

UNICODE_KEY = 'Unicode Key \N{TRADE MARK SIGN}'
UNICODE_VALUE = 'Unicode Value ' \
                '\N{COPYRIGHT SIGN},\N{TRADE MARK SIGN},\N{REGISTERED SIGN}'
FAKE_KEY = 'SOFTWARE\\{0}'.format(generate_random_name('SaltTesting-'))


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not salt.utils.platform.is_windows(), 'System is not Windows')
class WinFunctionsTestCase(TestCase):
    '''
    Test cases for salt.utils.win_reg
    '''
    def test_broadcast_change_success(self):
        '''
        Tests the broadcast_change function
        '''
        with patch('win32gui.SendMessageTimeout', return_value=('', 0)):
            self.assertEqual(win_reg.broadcast_change(), True)

    def test_broadcast_change_fail(self):
        '''
        Tests the broadcast_change function failure
        '''
        with patch('win32gui.SendMessageTimeout', return_value=('', 1)):
            self.assertEqual(win_reg.broadcast_change(), False)

    def test_key_exists_existing(self):
        '''
        Tests the key exists function using a well known registry key
        '''
        self.assertEqual(
            win_reg.key_exists(
                hive='HKLM',
                key='SOFTWARE\\Microsoft'
            ),
            True
        )

    def test_key_exists_non_existing(self):
        '''
        Tests the key exists function using a non existing registry key
        '''
        self.assertEqual(
            win_reg.key_exists(
                hive='HKLM',
                key=FAKE_KEY
            ),
            False
        )

    def test_list_keys_existing(self):
        '''
        Test the list_keys function using a well known registry key
        '''
        self.assertIn(
            'Microsoft',
            win_reg.list_keys(
                hive='HKLM',
                key='SOFTWARE'
            )
        )

    def test_list_keys_non_existing(self):
        '''
        Test the list_keys function using a non existing registry key
        '''
        expected = (False, 'Cannot find key: HKLM\\{0}'.format(FAKE_KEY))
        self.assertEqual(
            win_reg.list_keys(
                hive='HKLM',
                key=FAKE_KEY
            ),
            expected
        )

    def test_list_values_existing(self):
        '''
        Test the list_values function using a well known registry key
        '''
        values = win_reg.list_values(
            hive='HKLM',
            key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion'
        )
        keys = []
        for value in values:
            keys.append(value['vname'])
        self.assertIn('ProgramFilesDir', keys)

    def test_list_values_non_existing(self):
        '''
        Test the list_values function using a non existing registry key
        '''
        expected = (False, 'Cannot find key: HKLM\\{0}'.format(FAKE_KEY))
        self.assertEqual(
            win_reg.list_values(
                hive='HKLM',
                key=FAKE_KEY
            ),
            expected
        )

    def test_read_value_existing(self):
        '''
        Test the read_value function using a well known registry value
        '''
        ret = win_reg.read_value(
            hive='HKLM',
            key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion',
            vname='ProgramFilesPath'
        )
        self.assertEqual(ret['vdata'], '%ProgramFiles%')

    def test_read_value_default(self):
        '''
        Test the read_value function reading the default value using a well
        known registry key
        '''
        ret = win_reg.read_value(
            hive='HKLM',
            key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion'
        )
        self.assertEqual(ret['vdata'], '(value not set)')

    def test_read_value_non_existing(self):
        '''
        Test the read_value function using a non existing value pair
        '''
        expected = {
            'comment': 'Cannot find fake_name in HKLM\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion',
            'vdata': None,
            'vname': 'fake_name',
            'success': False,
            'hive': 'HKLM',
            'key': 'SOFTWARE\\Microsoft\\Windows\\CurrentVersion'
        }
        self.assertEqual(
            win_reg.read_value(
                hive='HKLM',
                key='SOFTWARE\\Microsoft\\Windows\\CurrentVersion',
                vname='fake_name'
            ),
            expected
        )

    def test_read_value_non_existing_key(self):
        '''
        Test the read_value function using a non existing registry key
        '''
        expected = {
            'comment': 'Cannot find key: HKLM\\{0}'.format(FAKE_KEY),
            'vdata': None,
            'vname': 'fake_name',
            'success': False,
            'hive': 'HKLM',
            'key': FAKE_KEY
        }
        self.assertEqual(
            win_reg.read_value(
                hive='HKLM',
                key=FAKE_KEY,
                vname='fake_name'
            ),
            expected
        )

    @destructiveTest
    def test_set_value(self):
        '''
        Test the set_value function
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_name',
                    vdata='fake_data'
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': FAKE_KEY,
                'success': True,
                'vdata': 'fake_data',
                'vname': 'fake_name',
                'vtype': 'REG_SZ'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_name'
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_set_value_default(self):
        '''
        Test the set_value function on the default value
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vdata='fake_default_data'
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': FAKE_KEY,
                'success': True,
                'vdata': 'fake_default_data',
                'vname': '(Default)',
                'vtype': 'REG_SZ'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_set_value_unicode_key(self):
        '''
        Test the set_value function on a unicode key
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key='{0}\\{1}'.format(FAKE_KEY, UNICODE_KEY),
                    vname='fake_name',
                    vdata='fake_value'
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': '{0}\\{1}'.format(FAKE_KEY, UNICODE_KEY),
                'success': True,
                'vdata': 'fake_value',
                'vname': 'fake_name',
                'vtype': 'REG_SZ'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key='{0}\\{1}'.format(FAKE_KEY, UNICODE_KEY),
                    vname='fake_name'
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_set_value_unicode_value(self):
        '''
        Test the set_value function on a unicode value
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_unicode',
                    vdata=UNICODE_VALUE
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': FAKE_KEY,
                'success': True,
                'vdata': UNICODE_VALUE,
                'vname': 'fake_unicode',
                'vtype': 'REG_SZ'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_unicode'
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_set_value_reg_dword(self):
        '''
        Test the set_value function on a unicode value
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='dword_value',
                    vdata=123,
                    vtype='REG_DWORD'
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': FAKE_KEY,
                'success': True,
                'vdata': 123,
                'vname': 'dword_value',
                'vtype': 'REG_DWORD'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='dword_value'
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_set_value_reg_qword(self):
        '''
        Test the set_value function on a unicode value
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='qword_value',
                    vdata=123,
                    vtype='REG_QWORD'
                )
            )
            expected = {
                'hive': 'HKLM',
                'key': FAKE_KEY,
                'success': True,
                'vdata': 123,
                'vname': 'qword_value',
                'vtype': 'REG_QWORD'
            }
            self.assertEqual(
                win_reg.read_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='qword_value'
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_delete_value(self):
        '''
        Test the delete_value function
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_name',
                    vdata='fake_data'
                )
            )
            self.assertTrue(
                win_reg.delete_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_name'
                )
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    def test_delete_value_non_existing(self):
        '''
        Test the delete_value function on non existing value
        '''
        self.assertEqual(
            win_reg.delete_value(
                hive='HKLM',
                key=FAKE_KEY,
                vname='fake_name'
            ),
            None
        )

    @destructiveTest
    def test_delete_value_unicode(self):
        '''
        Test the delete_value function on a unicode value
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_unicode',
                    vdata=UNICODE_VALUE
                )
            )
            self.assertTrue(
                win_reg.delete_value(
                    hive='HKLM',
                    key=FAKE_KEY,
                    vname='fake_unicode'
                )
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)

    @destructiveTest
    def test_delete_key_unicode(self):
        '''
        Test the delete_value function on value within a unicode key
        '''
        try:
            self.assertTrue(
                win_reg.set_value(
                    hive='HKLM',
                    key='{0}\\{1}'.format(FAKE_KEY, UNICODE_KEY),
                    vname='fake_name',
                    vdata='fake_value'
                )
            )
            expected = {
                'Deleted': ['HKLM\\{0}\\{1}\\'.format(FAKE_KEY, UNICODE_KEY)],
                'Failed': []
            }
            self.assertEqual(
                win_reg.delete_key_recursive(
                    hive='HKLM',
                    key='{0}\\{1}\\'.format(FAKE_KEY, UNICODE_KEY),
                ),
                expected
            )
        finally:
            win_reg.delete_key_recursive(hive='HKLM', key=FAKE_KEY)



# -*- coding: utf-8 -*-
'''
Integration tests for the Dimension Data cloud provider
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import random
import string

# Import Salt Testing Libs
from tests.support.case import ShellCase
from tests.support.paths import FILES
from tests.support.helpers import expensiveTest

# Import Salt Libs
from salt.config import cloud_providers_config
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin


def _random_name(size=6):
    '''
    Generates a random cloud instance name
    '''
    return 'cloud-test-' + ''.join(
        random.choice(string.ascii_lowercase + string.digits)
        for x in range(size)
    )


# Create the cloud instance name to be used throughout the tests
INSTANCE_NAME = _random_name()
PROVIDER_NAME = 'dimensiondata'


class DimensionDataTest(ShellCase):
    '''
    Integration tests for the Dimension Data cloud provider in Salt-Cloud
    '''

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''
        super(DimensionDataTest, self).setUp()

        # check if appropriate cloud provider and profile files are present
        profile_str = 'dimensiondata-config'
        providers = self.run_cloud('--list-providers')
        if profile_str + ':' not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf files '
                'in tests/integration/files/conf/cloud.*.d/ to run these tests.'
                .format(PROVIDER_NAME)
            )

        # check if user_id, key, and region are present
        config = cloud_providers_config(
            os.path.join(
                FILES,
                'conf',
                'cloud.providers.d',
                PROVIDER_NAME + '.conf'
            )
        )

        user_id = config[profile_str][PROVIDER_NAME]['user_id']
        key = config[profile_str][PROVIDER_NAME]['key']
        region = config[profile_str][PROVIDER_NAME]['region']

        if user_id == '' or key == '' or region == '':
            self.skipTest(
                'A user Id, password, and a region '
                'must be provided to run these tests. Check '
                'tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(PROVIDER_NAME)
            )

    def test_list_images(self):
        '''
        Tests the return of running the --list-images command for the dimensiondata cloud provider
        '''
        image_list = self.run_cloud('--list-images {0}'.format(PROVIDER_NAME))
        self.assertIn(
            'Ubuntu 14.04 2 CPU',
            [i.strip() for i in image_list]
        )

    def test_list_locations(self):
        '''
        Tests the return of running the --list-locations command for the dimensiondata cloud provider
        '''
        _list_locations = self.run_cloud('--list-locations {0}'.format(PROVIDER_NAME))
        self.assertIn(
            'Australia - Melbourne MCP2',
            [i.strip() for i in _list_locations]
        )

    def test_list_sizes(self):
        '''
        Tests the return of running the --list-sizes command for the dimensiondata cloud provider
        '''
        _list_sizes = self.run_cloud('--list-sizes {0}'.format(PROVIDER_NAME))
        self.assertIn(
            'default',
            [i.strip() for i in _list_sizes]
        )

    def test_instance(self):
        '''
        Test creating an instance on Dimension Data's cloud
        '''
        # check if instance with salt installed returned
        try:
            self.assertIn(
                INSTANCE_NAME,
                [i.strip() for i in self.run_cloud('-p dimensiondata-test {0}'.format(INSTANCE_NAME), timeout=500)]
            )
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)
            raise

        # delete the instance
        try:
            self.assertIn(
                'True',
                [i.strip() for i in self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)]
            )
        except AssertionError:
            raise

        # Final clean-up of created instance, in case something went wrong.
        # This was originally in a tearDown function, but that didn't make sense
        # To run this for each test when not all tests create instances.
        if INSTANCE_NAME in [i.strip() for i in self.run_cloud('--query')]:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)



# -*- coding: utf-8 -*-
'''
Test the ssh_known_hosts states
'''

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function
import os
import shutil

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.helpers import skip_if_binaries_missing

KNOWN_HOSTS = os.path.join(RUNTIME_VARS.TMP, 'known_hosts')
GITHUB_FINGERPRINT = '9d:38:5b:83:a9:17:52:92:56:1a:5e:c4:d4:81:8e:0a:ca:51:a2:64:f1:74:20:11:2e:f8:8a:c3:a1:39:49:8f'
GITHUB_IP = '192.30.253.113'


@skip_if_binaries_missing(['ssh', 'ssh-keygen'], check_all=True)
class SSHKnownHostsStateTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the ssh state
    '''
    def tearDown(self):
        if os.path.isfile(KNOWN_HOSTS):
            os.remove(KNOWN_HOSTS)
        super(SSHKnownHostsStateTest, self).tearDown()

    def test_present(self):
        '''
        ssh_known_hosts.present
        '''
        kwargs = {
            'name': 'github.com',
            'user': 'root',
            'fingerprint': GITHUB_FINGERPRINT,
            'config': KNOWN_HOSTS
        }
        # test first
        ret = self.run_state('ssh_known_hosts.present', test=True, **kwargs)
        self.assertSaltNoneReturn(ret)

        # save once, new key appears
        ret = self.run_state('ssh_known_hosts.present', **kwargs)
        try:
            self.assertSaltTrueReturn(ret)
        except AssertionError as err:
            try:
                self.assertInSaltComment(
                    'Unable to receive remote host key', ret
                )
                self.skipTest('Unable to receive remote host key')
            except AssertionError:
                # raise initial assertion error
                raise err

        self.assertSaltStateChangesEqual(
            ret, GITHUB_FINGERPRINT, keys=('new', 0, 'fingerprint')
        )

        # save twice, no changes
        self.run_state('ssh_known_hosts.present', **kwargs)

        # test again, nothing is about to be changed
        ret = self.run_state('ssh_known_hosts.present', test=True, **kwargs)
        self.assertSaltTrueReturn(ret)

        # then add a record for IP address
        ret = self.run_state('ssh_known_hosts.present',  # pylint: disable=repeated-keyword
                             **dict(kwargs, name=GITHUB_IP))
        try:
            self.assertSaltStateChangesEqual(
                ret, GITHUB_FINGERPRINT, keys=('new', 0, 'fingerprint')
            )
        except AssertionError as err:
            try:
                self.assertInSaltComment(
                        'Unable to receive remote host key', ret
                        )
                self.skipTest('Unable to receive remote host key')
            except AssertionError:
                raise err

        # record for every host must be available
        ret = self.run_function(
            'ssh.get_known_host_entries', ['root', 'github.com'], config=KNOWN_HOSTS
        )[0]
        try:
            self.assertNotIn(ret, ('', None))
        except AssertionError:
            raise AssertionError(
                'Salt return \'{0}\' is in (\'\', None).'.format(ret)
            )
        ret = self.run_function(
            'ssh.get_known_host_entries', ['root', GITHUB_IP], config=KNOWN_HOSTS
        )[0]
        try:
            self.assertNotIn(ret, ('', None, {}))
        except AssertionError:
            raise AssertionError(
                'Salt return \'{0}\' is in (\'\', None,'.format(ret) + ' {})'
            )

    def test_present_fail(self):
        # save something wrong
        ret = self.run_state(
            'ssh_known_hosts.present',
            name='github.com',
            user='root',
            fingerprint='aa:bb:cc:dd',
            config=KNOWN_HOSTS
        )
        self.assertSaltFalseReturn(ret)

    def test_absent(self):
        '''
        ssh_known_hosts.absent
        '''
        known_hosts = os.path.join(RUNTIME_VARS.FILES, 'ssh', 'known_hosts')
        shutil.copyfile(known_hosts, KNOWN_HOSTS)
        if not os.path.isfile(KNOWN_HOSTS):
            self.skipTest(
                'Unable to copy {0} to {1}'.format(
                    known_hosts, KNOWN_HOSTS
                )
            )

        kwargs = {'name': 'github.com', 'user': 'root', 'config': KNOWN_HOSTS}
        # test first
        ret = self.run_state('ssh_known_hosts.absent', test=True, **kwargs)
        self.assertSaltNoneReturn(ret)

        # remove once, the key is gone
        ret = self.run_state('ssh_known_hosts.absent', **kwargs)
        self.assertSaltStateChangesEqual(
            ret, GITHUB_FINGERPRINT, keys=('old', 0, 'fingerprint')
        )

        # remove twice, nothing has changed
        ret = self.run_state('ssh_known_hosts.absent', **kwargs)
        self.assertSaltStateChangesEqual(ret, {})

        # test again
        ret = self.run_state('ssh_known_hosts.absent', test=True, **kwargs)
        self.assertSaltTrueReturn(ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.unit import TestCase, expectedFailure


class SimpleTest(TestCase):
    def test_success(self):
        assert True

    @expectedFailure
    def test_fail(self):
        assert False



# -*- coding: utf-8 -*-
'''
Tests for salt.states.zfs

:codeauthor:    Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs, salt.modules.zfs
:platform:      illumos,freebsd,linux
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import test data from salt.utils.zfs test
from tests.unit.utils.test_zfs import utils_patch

# Import Salt Execution module to test
import salt.utils.zfs
import salt.states.zfs as zfs

# Import Salt Utils
import salt.loader
from salt.utils.odict import OrderedDict


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZfsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.zfs
    '''
    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS
        utils = salt.loader.utils(opts, whitelist=['zfs'])
        zfs_obj = {
            zfs: {
                '__opts__': opts,
                '__grains__': {'kernel': 'SunOS'},
                '__utils__': utils,
            }
        }

        return zfs_obj

    def test_filesystem_absent_nofs(self):
        '''
        Test if filesystem is absent (non existing filesystem)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'filesystem myzpool/filesystem is absent',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_absent('myzpool/filesystem'))

    def test_filesystem_absent_removed(self):
        '''
        Test if filesystem is absent
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'filesystem myzpool/filesystem was destroyed',
               'changes': {'myzpool/filesystem': 'destroyed'}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([('destroyed', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_absent('myzpool/filesystem'))

    def test_filesystem_absent_fail(self):
        '''
        Test if filesystem is absent (with snapshots)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': False,
               'comment': "\n".join([
                    "cannot destroy 'myzpool/filesystem': filesystem has children",
                    "use 'recursive=True' to destroy the following datasets:",
                    "myzpool/filesystem@snap",
                ]),
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('destroyed', False),
            ('error', "\n".join([
                "cannot destroy 'myzpool/filesystem': filesystem has children",
                "use 'recursive=True' to destroy the following datasets:",
                "myzpool/filesystem@snap",
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_absent('myzpool/filesystem'))

    def test_volume_absent_novol(self):
        '''
        Test if volume is absent (non existing volume)
        '''
        ret = {'name': 'myzpool/volume',
               'result': True,
               'comment': 'volume myzpool/volume is absent',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_absent('myzpool/volume'))

    def test_volume_absent_removed(self):
        '''
        Test if volume is absent
        '''
        ret = {'name': 'myzpool/volume',
               'result': True,
               'comment': 'volume myzpool/volume was destroyed',
               'changes': {'myzpool/volume': 'destroyed'}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([('destroyed', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_absent('myzpool/volume'))

    def test_volume_absent_fail(self):
        '''
        Test if volume is absent (with snapshots)
        '''
        ret = {'name': 'myzpool/volume',
               'result': False,
               'comment': "\n".join([
                    "cannot destroy 'myzpool/volume': volume has children",
                    "use 'recursive=True' to destroy the following datasets:",
                    "myzpool/volume@snap",
                ]),
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('destroyed', False),
            ('error', "\n".join([
                "cannot destroy 'myzpool/volume': volume has children",
                "use 'recursive=True' to destroy the following datasets:",
                "myzpool/volume@snap",
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_absent('myzpool/volume'))

    def test_snapshot_absent_nosnap(self):
        '''
        Test if snapshot is absent (non existing snapshot)
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': True,
               'comment': 'snapshot myzpool/filesystem@snap is absent',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_absent('myzpool/filesystem@snap'))

    def test_snapshot_absent_removed(self):
        '''
        Test if snapshot is absent
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': True,
               'comment': 'snapshot myzpool/filesystem@snap was destroyed',
               'changes': {'myzpool/filesystem@snap': 'destroyed'}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([('destroyed', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_absent('myzpool/filesystem@snap'))

    def test_snapshot_absent_fail(self):
        '''
        Test if snapshot is absent (with snapshots)
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': False,
               'comment': 'cannot destroy snapshot myzpool/filesystem@snap: dataset is busy',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([
            ('destroyed', False),
            ('error', 'cannot destroy snapshot myzpool/filesystem@snap: dataset is busy'),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_absent('myzpool/filesystem@snap'))

    def test_bookmark_absent_nobook(self):
        '''
        Test if bookmark is absent (non existing bookmark)
        '''
        ret = {'name': 'myzpool/filesystem#book',
               'result': True,
               'comment': 'bookmark myzpool/filesystem#book is absent',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.bookmark_absent('myzpool/filesystem#book'))

    def test_bookmark_absent_removed(self):
        '''
        Test if bookmark is absent
        '''
        ret = {'name': 'myzpool/filesystem#book',
               'result': True,
               'comment': 'bookmark myzpool/filesystem#book was destroyed',
               'changes': {'myzpool/filesystem#book': 'destroyed'}}

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([('destroyed', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.destroy': mock_destroy}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.bookmark_absent('myzpool/filesystem#book'))

    def test_hold_absent_nohold(self):
        '''
        Test if hold is absent (non existing hold)
        '''
        ret = {'name': 'myhold',
               'result': True,
               'comment': 'hold myhold is absent',
               'changes': {}}

        mock_holds = MagicMock(return_value=OrderedDict([]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_absent('myhold', 'myzpool/filesystem@snap'))

    def test_hold_absent_removed(self):
        '''
        Test if hold is absent
        '''
        ret = {'name': 'myhold',
               'result': True,
               'comment': 'hold myhold released',
               'changes': OrderedDict([
                    ('myzpool/filesystem@snap', OrderedDict([
                        ('myhold', 'released'),
                    ])),
                ])}

        mock_holds = MagicMock(return_value=OrderedDict([('myhold', 'Thu Feb 15 16:24 2018')]))
        mock_release = MagicMock(return_value=OrderedDict([('released', True)]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__salt__, {'zfs.release': mock_release}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_absent('myhold', 'myzpool/filesystem@snap'))

    def test_hold_absent_fail(self):
        '''
        Test if hold is absent (non existing snapshot)
        '''
        ret = {'name': 'myhold',
               'result': False,
               'comment': "cannot open 'myzpool/filesystem@snap': dataset does not exist",
               'changes': {}}

        mock_holds = MagicMock(return_value=OrderedDict([
            ('error', "cannot open 'myzpool/filesystem@snap': dataset does not exist"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_absent('myhold', 'myzpool/filesystem@snap'))

    def test_hold_present(self):
        '''
        Test if hold is present (hold already present)
        '''
        ret = {'name': 'myhold',
               'result': True,
               'comment': 'hold myhold is present for myzpool/filesystem@snap',
               'changes': {}}

        mock_holds = MagicMock(return_value=OrderedDict([('myhold', 'Thu Feb 15 16:24 2018')]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_present('myhold', 'myzpool/filesystem@snap'))

    def test_hold_present_new(self):
        '''
        Test if hold is present (new)
        '''
        ret = {'name': 'myhold',
               'result': True,
               'comment': 'hold myhold added to myzpool/filesystem@snap',
               'changes': {'myzpool/filesystem@snap': {'myhold': 'held'}}}

        mock_holds = MagicMock(return_value=OrderedDict([]))
        mock_hold = MagicMock(return_value=OrderedDict([('held', True)]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__salt__, {'zfs.hold': mock_hold}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_present('myhold', 'myzpool/filesystem@snap'))

    def test_hold_present_fail(self):
        '''
        Test if hold is present (using non existing snapshot)
        '''
        ret = {'name': 'myhold',
               'result': False,
               'comment': "cannot hold snapshot 'zsalt/filesystem@snap': dataset does not exist",
               'changes': {}}

        mock_holds = MagicMock(return_value=OrderedDict([]))
        mock_hold = MagicMock(return_value=OrderedDict([
            ('held', False),
            ('error', "cannot hold snapshot 'zsalt/filesystem@snap': dataset does not exist"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.holds': mock_holds}), \
             patch.dict(zfs.__salt__, {'zfs.hold': mock_hold}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.hold_present('myhold', 'myzpool/filesystem@snap'))

    def test_filesystem_present(self):
        '''
        Test if filesystem is present (existing filesystem)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'filesystem myzpool/filesystem is uptodate',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/filesystem', OrderedDict([
                ('type', OrderedDict([
                    ('value', 'filesystem'),
                ])),
                ('compression', OrderedDict([
                    ('value', False),
                ])),
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_present('myzpool/filesystem'))

    def test_filesystem_present_new(self):
        '''
        Test if filesystem is present (non existing filesystem)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'filesystem myzpool/filesystem was created',
               'changes': {'myzpool/filesystem': u'created'}}

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([('created', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.create': mock_create}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_present('myzpool/filesystem'))

    def test_filesystem_present_update(self):
        '''
        Test if filesystem is present (non existing filesystem)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'filesystem myzpool/filesystem was updated',
               'changes': {'myzpool/filesystem': {'compression': 'lz4'}}}

        mock_exists = MagicMock(return_value=True)
        mock_set = MagicMock(return_value=OrderedDict([('set', True)]))
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/filesystem', OrderedDict([
                ('type', OrderedDict([
                    ('value', 'filesystem'),
                ])),
                ('compression', OrderedDict([
                    ('value', False),
                ])),
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__salt__, {'zfs.set': mock_set}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_present(
                name='myzpool/filesystem',
                properties={'compression': 'lz4'},
            ))

    def test_filesystem_present_fail(self):
        '''
        Test if filesystem is present (non existing pool)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': False,
               'comment': "cannot create 'myzpool/filesystem': no such pool 'myzpool'",
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([
            ('created', False),
            ('error', "cannot create 'myzpool/filesystem': no such pool 'myzpool'"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.create': mock_create}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.filesystem_present('myzpool/filesystem'))

    def test_volume_present(self):
        '''
        Test if volume is present (existing volume)
        '''
        ret = {'name': 'myzpool/volume',
               'result': True,
               'comment': 'volume myzpool/volume is uptodate',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/volume', OrderedDict([
                ('type', OrderedDict([
                    ('value', 'volume'),
                ])),
                ('compression', OrderedDict([
                    ('value', False),
                ])),
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_present('myzpool/volume', volume_size='1G'))

    def test_volume_present_new(self):
        '''
        Test if volume is present (non existing volume)
        '''
        ret = {'name': 'myzpool/volume',
               'result': True,
               'comment': 'volume myzpool/volume was created',
               'changes': {'myzpool/volume': u'created'}}

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([('created', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.create': mock_create}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_present('myzpool/volume', volume_size='1G'))

    def test_volume_present_update(self):
        '''
        Test if volume is present (non existing volume)
        '''
        ret = {'name': 'myzpool/volume',
               'result': True,
               'comment': 'volume myzpool/volume was updated',
               'changes': {'myzpool/volume': {'compression': 'lz4'}}}

        mock_exists = MagicMock(return_value=True)
        mock_set = MagicMock(return_value=OrderedDict([('set', True)]))
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/volume', OrderedDict([
                ('type', OrderedDict([
                    ('value', 'volume'),
                ])),
                ('compression', OrderedDict([
                    ('value', False),
                ])),
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__salt__, {'zfs.set': mock_set}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_present(
                name='myzpool/volume',
                volume_size='1G',
                properties={'compression': 'lz4'},
            ))

    def test_volume_present_fail(self):
        '''
        Test if volume is present (non existing pool)
        '''
        ret = {'name': 'myzpool/volume',
               'result': False,
               'comment': "cannot create 'myzpool/volume': no such pool 'myzpool'",
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(return_value=OrderedDict([
            ('created', False),
            ('error', "cannot create 'myzpool/volume': no such pool 'myzpool'"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.create': mock_create}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.volume_present('myzpool/volume', volume_size='1G'))

    def test_bookmark_present(self):
        '''
        Test if bookmark is present (bookmark already present)
        '''
        ret = {'name': 'myzpool/filesystem#mybookmark',
               'result': True,
               'comment': 'bookmark is present',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.bookmark_present('mybookmark', 'myzpool/filesystem@snap'))

    def test_bookmark_present_new(self):
        '''
        Test if bookmark is present (new)
        '''
        ret = {'name': 'myzpool/filesystem#mybookmark',
               'result': True,
               'comment': 'myzpool/filesystem@snap bookmarked as myzpool/filesystem#mybookmark',
               'changes': {'myzpool/filesystem#mybookmark': 'myzpool/filesystem@snap'}}

        mock_exists = MagicMock(return_value=False)
        mock_bookmark = MagicMock(return_value=OrderedDict([('bookmarked', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.bookmark': mock_bookmark}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.bookmark_present('mybookmark', 'myzpool/filesystem@snap'))

    def test_bookmark_present_fail(self):
        '''
        Test if bookmark is present (using non existing snapshot)
        '''
        ret = {'name': 'myzpool/filesystem#mybookmark',
               'result': False,
               'comment': "cannot bookmark snapshot 'zsalt/filesystem@snap': dataset does not exist",
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        mock_bookmark = MagicMock(return_value=OrderedDict([
            ('bookmarked', False),
            ('error', "cannot bookmark snapshot 'zsalt/filesystem@snap': dataset does not exist"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.bookmark': mock_bookmark}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.bookmark_present('mybookmark', 'myzpool/filesystem@snap'))

    def test_snapshot_present(self):
        '''
        Test if snapshot is present (snapshot already present)
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': True,
               'comment': 'snapshot is present',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_present('myzpool/filesystem@snap'))

    def test_snapshot_present_new(self):
        '''
        Test if snapshot is present (new)
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': True,
               'comment': 'snapshot myzpool/filesystem@snap was created',
               'changes': {u'myzpool/filesystem@snap': u'snapshotted'}}

        mock_exists = MagicMock(return_value=False)
        mock_snapshot = MagicMock(return_value=OrderedDict([('snapshotted', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.snapshot': mock_snapshot}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_present('myzpool/filesystem@snap'))

    def test_snapshot_present_fail(self):
        '''
        Test if snapshot is present (using non existing snapshot)
        '''
        ret = {'name': 'myzpool/filesystem@snap',
               'result': False,
               'comment': "cannot open 'myzpool/filesystem': dataset does not exist",
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        mock_snapshot = MagicMock(return_value=OrderedDict([
            ('snapshotted', False),
            ('error', "cannot open 'myzpool/filesystem': dataset does not exist"),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.snapshot': mock_snapshot}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.snapshot_present('myzpool/filesystem@snap'))

    def test_propmoted(self):
        '''
        Test promotion of clone (already promoted)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'myzpool/filesystem already promoted',
               'changes': {}}

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/filesystem', OrderedDict([
                ('origin', OrderedDict([
                    ('value', '-'),
                ])),
            ])),
        ]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.promoted('myzpool/filesystem'))

    def test_propmoted_clone(self):
        '''
        Test promotion of clone
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': True,
               'comment': 'myzpool/filesystem promoted',
               'changes': {'myzpool/filesystem': 'promoted'}}

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(return_value=OrderedDict([
            ('myzpool/filesystem', OrderedDict([
                ('origin', OrderedDict([
                    ('value', 'myzool/filesystem_source@clean'),
                ])),
            ])),
        ]))
        mock_promote = MagicMock(return_value=OrderedDict([('promoted', True)]))
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__salt__, {'zfs.get': mock_get}), \
             patch.dict(zfs.__salt__, {'zfs.promote': mock_promote}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.promoted('myzpool/filesystem'))

    def test_propmoted_fail(self):
        '''
        Test promotion of clone (unknown dataset)
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': False,
               'comment': 'dataset myzpool/filesystem does not exist',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.promoted('myzpool/filesystem'))

    def test_scheduled_snapshot_fail(self):
        '''
        Test scheduled_snapshot of unknown dataset
        '''
        ret = {'name': 'myzpool/filesystem',
               'result': False,
               'comment': 'dataset myzpool/filesystem does not exist',
               'changes': {}}

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zfs.__salt__, {'zfs.exists': mock_exists}), \
             patch.dict(zfs.__utils__, utils_patch):
            self.assertEqual(ret, zfs.scheduled_snapshot('myzpool/filesystem', 'shadow', schedule={'hour': 6}))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)

# Import Salt Libs
import salt.modules.rdp as rdp


class RdpTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.rdp
    '''
    def setup_loader_modules(self):
        return {rdp: {}}

    # 'enable' function tests: 1

    def test_enable(self):
        '''
        Test if it enables RDP the service on the server
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(rdp.__salt__, {'cmd.run': mock}), \
                patch('salt.modules.rdp._parse_return_code_powershell',
                      MagicMock(return_value=0)):
            self.assertTrue(rdp.enable())

    # 'disable' function tests: 1

    def test_disable(self):
        '''
        Test if it disables RDP the service on the server
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(rdp.__salt__, {'cmd.run': mock}), \
                patch('salt.modules.rdp._parse_return_code_powershell',
                      MagicMock(return_value=0)):
            self.assertTrue(rdp.disable())

    # 'status' function tests: 1

    def test_status(self):
        '''
        Test if it shows rdp is enabled on the server
        '''
        mock = MagicMock(return_value='1')
        with patch.dict(rdp.__salt__, {'cmd.run': mock}):
            self.assertTrue(rdp.status())



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

# Import Salt Libs
import salt.states.rbenv as rbenv


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RbenvTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.rbenv
    '''
    def setup_loader_modules(self):
        return {rbenv: {}}

    # 'installed' function tests: 1

    def test_installed(self):
        '''
        Test to verify that the specified ruby is installed with rbenv.
        '''
        # rbenv.is_installed is used wherever test is False.
        mock_is = MagicMock(side_effect=[False, True, True, True, True])

        # rbenv.install is only called when an action is attempted
        # (ie. Successfully... or Failed...)
        mock_i = MagicMock(side_effect=[False, False, False])

        # rbenv.install_ruby is only called when rbenv is successfully
        # installed and an attempt to install a version of Ruby is
        # made.
        mock_ir = MagicMock(side_effect=[True, False])
        mock_def = MagicMock(return_value='2.3.4')
        mock_ver = MagicMock(return_value=['2.3.4', '2.4.1'])
        with patch.dict(rbenv.__salt__,
                        {'rbenv.is_installed': mock_is,
                         'rbenv.install': mock_i,
                         'rbenv.default': mock_def,
                         'rbenv.versions': mock_ver,
                         'rbenv.install_ruby': mock_ir}):
            with patch.dict(rbenv.__opts__, {'test': True}):
                name = '1.9.3-p551'
                comt = 'Ruby {0} is set to be installed'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': None}
                self.assertDictEqual(rbenv.installed(name), ret)

                name = '2.4.1'
                comt = 'Ruby {0} is already installed'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': False, 'result': True}
                self.assertDictEqual(rbenv.installed(name), ret)

                name = '2.3.4'
                comt = 'Ruby {0} is already installed'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': True, 'result': True}
                self.assertDictEqual(rbenv.installed(name), ret)

            with patch.dict(rbenv.__opts__, {'test': False}):
                name = '2.4.1'
                comt = 'Rbenv failed to install'
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': False}
                self.assertDictEqual(rbenv.installed(name), ret)

                comt = 'Requested ruby exists'
                ret = {'name': name, 'comment': comt, 'default': False,
                       'changes': {}, 'result': True}
                self.assertDictEqual(rbenv.installed(name), ret)

                name = '2.3.4'
                comt = 'Requested ruby exists'
                ret = {'name': name, 'comment': comt, 'default': True,
                       'changes': {}, 'result': True}
                self.assertDictEqual(rbenv.installed(name), ret)

                name = '1.9.3-p551'
                comt = 'Successfully installed ruby'
                ret = {'name': name, 'comment': comt, 'default': False,
                       'changes': {name: 'Installed'}, 'result': True}
                self.assertDictEqual(rbenv.installed(name), ret)

                comt = 'Failed to install ruby'
                ret = {'name': name, 'comment': comt,
                       'changes': {}, 'result': False}
                self.assertDictEqual(rbenv.installed(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to verify that the specified ruby is not installed with rbenv.
        '''
        # rbenv.is_installed is used for all tests here.
        mock_is = MagicMock(side_effect=[False, True, True, True, False,
                                         True, True, True, True, True])
        # rbenv.uninstall_ruby is only called when an action is
        # attempted (ie. Successfully... or Failed...)
        mock_uninstalled = MagicMock(side_effect=[True, False, False, True])
        mock_def = MagicMock(return_value='2.3.4')
        mock_ver = MagicMock(return_value=['2.3.4', '2.4.1'])
        with patch.dict(rbenv.__salt__,
                        {'rbenv.is_installed': mock_is,
                         'rbenv.default': mock_def,
                         'rbenv.versions': mock_ver,
                         'rbenv.uninstall_ruby': mock_uninstalled}):

            with patch.dict(rbenv.__opts__, {'test': True}):
                name = '1.9.3-p551'
                comt = 'Rbenv not installed, {0} not either'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

                comt = 'Ruby {0} is already uninstalled'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

                name = '2.3.4'
                comt = 'Ruby {0} is set to be uninstalled'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': True, 'result': None}
                self.assertDictEqual(rbenv.absent('2.3.4'), ret)

                name = '2.4.1'
                comt = 'Ruby {0} is set to be uninstalled'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': False, 'result': None}
                self.assertDictEqual(rbenv.absent('2.4.1'), ret)

            with patch.dict(rbenv.__opts__, {'test': False}):
                name = '1.9.3-p551'
                comt = 'Rbenv not installed, {0} not either'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

                comt = 'Ruby {0} is already absent'.format(name)
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

                name = '2.3.4'
                comt = 'Successfully removed ruby'
                ret = {'name': name, 'changes': {name: 'Uninstalled'},
                       'comment': comt, 'default': True, 'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

                comt = 'Failed to uninstall ruby'
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': True, 'result': False}
                self.assertDictEqual(rbenv.absent(name), ret)

                name = '2.4.1'
                comt = 'Failed to uninstall ruby'
                ret = {'name': name, 'changes': {}, 'comment': comt,
                       'default': False, 'result': False}
                self.assertDictEqual(rbenv.absent(name), ret)

                comt = 'Successfully removed ruby'
                ret = {'name': name, 'changes': {name: 'Uninstalled'},
                       'comment': comt, 'default': False, 'result': True}
                self.assertDictEqual(rbenv.absent(name), ret)

    # 'install_rbenv' function tests: 1

    def test_install_rbenv(self):
        '''
        Test to install rbenv if not installed.
        '''
        name = 'myqueue'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'comment': ''}

        mock_is = MagicMock(side_effect=[False, True, True, False, False])
        mock_i = MagicMock(side_effect=[False, True])
        with patch.dict(rbenv.__salt__,
                        {'rbenv.is_installed': mock_is,
                         'rbenv.install': mock_i}):

            with patch.dict(rbenv.__opts__, {'test': True}):
                comt = 'Rbenv is set to be installed'
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(rbenv.install_rbenv(name), ret)

                comt = 'Rbenv is already installed'
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(rbenv.install_rbenv(name), ret)

            with patch.dict(rbenv.__opts__, {'test': False}):
                comt = 'Rbenv is already installed'
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(rbenv.install_rbenv(name), ret)

                comt = 'Rbenv failed to install'
                ret.update({'comment': comt, 'result': False})
                self.assertDictEqual(rbenv.install_rbenv(name), ret)

                comt = 'Rbenv installed'
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(rbenv.install_rbenv(name), ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import datetime
import re

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, Mock, patch, call

# Import salt libs
import salt.modules.postgres as postgres
from salt.exceptions import SaltInvocationError

test_list_db_csv = (
    'Name,Owner,Encoding,Collate,Ctype,Access privileges,Tablespace\n'
    'template1,postgres,LATIN1,en_US,en_US'
    ',"{=c/postgres,postgres=CTc/postgres}",pg_default\n'
    'template0,postgres,LATIN1,en_US,en_US'
    ',"{=c/postgres,postgres=CTc/postgres}",pg_default\n'
    'postgres,postgres,LATIN1,en_US,en_US,,pg_default\n'
    'test_db,postgres,LATIN1,en_US,en_US,,pg_default'
)

test_list_schema_csv = (
    'name,owner,acl\n'
    'public,postgres,"{postgres=UC/postgres,=UC/postgres}"\n'
    'pg_toast,postgres,""'
)

test_list_language_csv = (
    'Name\n'
    'internal\n'
    'c\n'
    'sql\n'
    'plpgsql\n'
)

test_privileges_list_table_csv = (
    'name\n'
    '"{baruwatest=arwdDxt/baruwatest,bayestest=arwd/baruwatest,baruwa=a*r*w*d*D*x*t*/baruwatest}"\n'
)

test_privileges_list_group_csv = (
    'rolname,admin_option\n'
    'baruwa,f\n'
    'baruwatest2,t\n'
    'baruwatest,f\n'
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            postgres: {
                '__grains__': {'os_family': 'Linux'},
                '__salt__': {
                    'config.option': Mock(),
                    'cmd.run_all': Mock(),
                    'file.chown': Mock(),
                    'file.remove': Mock(),
                }
            }
        }

    def test_run_psql(self):
        postgres._run_psql('echo "hi"')
        cmd = postgres.__salt__['cmd.run_all']

        self.assertEqual('postgres', cmd.call_args[1]['runas'])

    def test_db_alter(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            postgres.db_alter('dbname',
                              user='testuser',
                              host='testhost',
                              port='testport',
                              maintenance_db='maint_db',
                              password='foo',
                              tablespace='testspace',
                              owner='otheruser',
                              runas='foo')
            postgres._run_psql.assert_has_calls([
                call(['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                      '--no-password', '--username', 'testuser', '--host',
                      'testhost', '--port', 'testport', '--dbname', 'maint_db',
                      '-c', 'ALTER DATABASE "dbname" OWNER TO "otheruser"'],
                     host='testhost', user='testuser',
                     password='foo', runas='foo', port='testport'),
                call(['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                      '--no-password', '--username', 'testuser', '--host',
                      'testhost', '--port', 'testport', '--dbname', 'maint_db',
                      '-c', 'ALTER DATABASE "dbname" SET TABLESPACE "testspace"'],
                     host='testhost', user='testuser',
                     password='foo', runas='foo', port='testport')
            ])

    def test_db_alter_owner_recurse(self):
        with patch('salt.modules.postgres.owner_to',
                   Mock(return_value={'retcode': None})):
            postgres.db_alter('dbname',
                              user='testuser',
                              host='testhost',
                              port='testport',
                              maintenance_db='maint_db',
                              password='foo',
                              tablespace='testspace',
                              owner='otheruser',
                              owner_recurse=True,
                              runas='foo')
            postgres.owner_to.assert_called_once_with('dbname',
                                                      'otheruser',
                                                      user='testuser',
                                                      host='testhost',
                                                      port='testport',
                                                      password='foo',
                                                      runas='foo')

    def test_db_create(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            postgres.db_create(
                'dbname',
                user='testuser',
                host='testhost',
                port='testport',
                maintenance_db='maint_db',
                password='foo',
                tablespace='testspace',
                owner='otheruser',
                runas='foo'
            )

            postgres._run_psql.assert_called_once_with(
                ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                 '--no-password', '--username', 'testuser', '--host',
                 'testhost', '--port', 'testport', '--dbname', 'maint_db',
                 '-c', 'CREATE DATABASE "dbname" WITH TABLESPACE = "testspace" '
                       'OWNER = "otheruser"'],
                host='testhost', user='testuser',
                password='foo', runas='foo', port='testport')

    def test_db_create_empty_string_param(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            postgres.db_create('dbname', lc_collate='', encoding='utf8',
                    user='testuser', host='testhost', port=1234,
                    maintenance_db='maint_db', password='foo')

            postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                        '--no-password', '--username', 'testuser', '--host',
                        'testhost', '--port', '1234', '--dbname', 'maint_db', '-c',
                        'CREATE DATABASE "dbname" WITH ENCODING = \'utf8\' '
                        'LC_COLLATE = \'\''], host='testhost', password='foo',
                    port=1234, runas=None, user='testuser')

    def test_db_create_with_trivial_sql_injection(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            self.assertRaises(
                    SaltInvocationError,
                    postgres.db_create,
                    'dbname', lc_collate="foo' ENCODING='utf8")

    def test_db_exists(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_list_db_csv})):
            ret = postgres.db_exists(
                'test_db',
                user='testuser',
                host='testhost',
                port='testport',
                maintenance_db='maint_db',
                password='foo',
                runas='foo'
            )
            self.assertTrue(ret)

    def test_db_list(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_list_db_csv})):
            ret = postgres.db_list(
                user='testuser',
                host='testhost',
                port='testport',
                maintenance_db='maint_db',
                password='foo',
                runas='foo'
            )
            self.assertDictEqual(ret, {
                'test_db': {'Encoding': 'LATIN1', 'Ctype': 'en_US',
                            'Tablespace': 'pg_default', 'Collate': 'en_US',
                            'Owner': 'postgres', 'Access privileges': ''},
                'template1': {'Encoding': 'LATIN1', 'Ctype': 'en_US',
                              'Tablespace': 'pg_default', 'Collate': 'en_US',
                              'Owner': 'postgres',
                              'Access privileges': (
                                  '{=c/postgres,postgres=CTc/postgres}'
                              )},
                'template0': {'Encoding': 'LATIN1', 'Ctype': 'en_US',
                              'Tablespace': 'pg_default', 'Collate': 'en_US',
                              'Owner': 'postgres',
                              'Access privileges': (
                                  '{=c/postgres,postgres=CTc/postgres}'
                              )},
                'postgres': {'Encoding': 'LATIN1', 'Ctype': 'en_US',
                             'Tablespace': 'pg_default', 'Collate': 'en_US',
                             'Owner': 'postgres', 'Access privileges': ''}})

    def test_db_remove(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            postgres.db_remove(
                'test_db',
                user='testuser',
                host='testhost',
                port='testport',
                maintenance_db='maint_db',
                password='foo',
                runas='foo'
            )
            postgres._run_psql.assert_called_once_with(
                ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                 '--no-password', '--username', 'testuser', '--host',
                 'testhost', '--port', 'testport', '--dbname', 'maint_db',
                 '-c', 'DROP DATABASE "test_db"'],
                host='testhost', user='testuser',
                password='foo', runas='foo', port='testport')

    def test_group_create(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.user_exists', Mock(return_value=False)):
                postgres.group_create(
                    'testgroup',
                    user='testuser',
                    host='testhost',
                    port='testport',
                    maintenance_db='maint_db',
                    password='foo',
                    createdb=False,
                    createuser=False,
                    encrypted=False,
                    superuser=False,
                    replication=False,
                    rolepassword='testrolepass',
                    groups='testgroup',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    postgres._run_psql.call_args[0][0][14].startswith('CREATE ROLE')
                )

    def test_group_remove(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.user_exists', Mock(return_value=True)):
                postgres.group_remove(
                    'testgroup',
                    user='testuser',
                    host='testhost',
                    port='testport',
                    maintenance_db='maint_db',
                    password='foo',
                    runas='foo'
                )
                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'maint_db',
                     '-c', 'DROP ROLE "testgroup"'],
                    host='testhost', user='testuser',
                    password='foo', runas='foo', port='testport')

    def test_group_update(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.role_get',
                       Mock(return_value={'superuser': False})):
                postgres.group_update(
                    'testgroup',
                    user='"testuser"',
                    host='testhost',
                    port='testport',
                    maintenance_db='maint_db',
                    password='foo',
                    createdb=False,
                    createuser=False,
                    encrypted=False,
                    replication=False,
                    rolepassword='test_role_pass',
                    groups='testgroup',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    re.match(
                        'ALTER.* "testgroup" .* UNENCRYPTED PASSWORD',
                        postgres._run_psql.call_args[0][0][14]
                    )
                )

    def test_user_create(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.user_exists', Mock(return_value=False)):
                postgres.user_create(
                    'testuser',
                    user='testuser',
                    host='testhost',
                    port='testport',
                    maintenance_db='maint_test',
                    password='test_pass',
                    login=True,
                    createdb=False,
                    createroles=False,
                    createuser=False,
                    encrypted=False,
                    superuser=False,
                    replication=False,
                    rolepassword='test_role_pass',
                    valid_until='2042-07-01',
                    groups='test_groups',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                call = postgres._run_psql.call_args[0][0][14]
                self.assertTrue(re.match('CREATE ROLE "testuser"', call))
                for i in (
                    'INHERIT', 'NOCREATEDB', 'NOCREATEROLE', 'NOSUPERUSER',
                    'NOREPLICATION', 'LOGIN', 'UNENCRYPTED', 'PASSWORD',
                    'VALID UNTIL',
                ):
                    self.assertTrue(i in call, '{0} not in {1}'.format(i, call))

    def test_user_exists(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.version',
                       Mock(return_value='9.1')):
                with patch('salt.modules.postgres.psql_query',
                           Mock(return_value=[
                               {
                                   'name': 'test_user',
                                   'superuser': 't',
                                   'inherits privileges': 't',
                                   'can create roles': 't',
                                   'can create databases': 't',
                                   'can update system catalogs': 't',
                                   'can login': 't',
                                   'replication': None,
                                   'password': 'test_password',
                                   'connections': '-1',
                                   'groups': '',
                                   'expiry time': '',
                                   'defaults variables': None
                                   }])):
                    ret = postgres.user_exists(
                        'test_user',
                        user='test_user',
                        host='test_host',
                        port='test_port',
                        maintenance_db='maint_db',
                        password='test_password',
                        runas='foo'
                    )
                    self.assertTrue(ret)

    def test_user_list(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.version',
                       Mock(return_value='9.1')):
                with patch('salt.modules.postgres.psql_query',
                           Mock(return_value=[
                               {
                                   'name': 'test_user',
                                   'superuser': 't',
                                   'inherits privileges': 't',
                                   'can create roles': 't',
                                   'can create databases': 't',
                                   'can update system catalogs': 't',
                                   'can login': 't',
                                   'replication': None,
                                   'connections': '-1',
                                   'groups': '',
                                   'expiry time': '2017-08-16 08:57:46',
                                   'defaults variables': None
                               }])):
                    ret = postgres.user_list(
                        'test_user',
                        host='test_host',
                        port='test_port',
                        maintenance_db='maint_db',
                        password='test_password',
                        runas='foo'
                    )

                    self.assertDictEqual(ret, {
                        'test_user': {'superuser': True,
                                      'defaults variables': None,
                                      'can create databases': True,
                                      'can create roles': True,
                                      'connections': None,
                                      'replication': None,
                                      'expiry time': datetime.datetime(
                                          2017, 8, 16, 8, 57, 46),
                                      'can login': True,
                                      'can update system catalogs': True,
                                      'groups': [],
                                      'inherits privileges': True}})

    def test_user_remove(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.version', Mock(return_value='9.1')):
                with patch('salt.modules.postgres.user_exists', Mock(return_value=True)):
                    postgres.user_remove(
                        'testuser',
                        user='testuser',
                        host='testhost',
                        port='testport',
                        maintenance_db='maint_db',
                        password='testpassword',
                        runas='foo'
                    )
                    postgres._run_psql.assert_called_once_with(
                        ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                         '--no-password', '--username', 'testuser', '--host',
                         'testhost', '--port', 'testport', '--dbname', 'maint_db',
                         '-c', 'DROP ROLE "testuser"'],
                        host='testhost', port='testport', user='testuser',
                        password='testpassword', runas='foo')

    def test_user_update(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.role_get',
                       Mock(return_value={'superuser': False})):
                postgres.user_update(
                    'test_username',
                    user='test_user',
                    host='test_host',
                    port='test_port',
                    maintenance_db='test_maint',
                    password='test_pass',
                    createdb=False,
                    createroles=False,
                    createuser=False,
                    encrypted=False,
                    inherit=True,
                    login=True,
                    replication=False,
                    rolepassword='test_role_pass',
                    valid_until='2017-07-01',
                    groups='test_groups',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    re.match(
                        'ALTER ROLE "test_username" WITH  INHERIT NOCREATEDB '
                        'NOCREATEROLE NOREPLICATION LOGIN '
                        'UNENCRYPTED PASSWORD [\'"]{0,5}test_role_pass[\'"]{0,5} '
                        'VALID UNTIL \'2017-07-01\';'
                        ' GRANT "test_groups" TO "test_username"',
                        postgres._run_psql.call_args[0][0][14]
                    )
                )

    def test_user_update2(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.role_get',
                       Mock(return_value={'superuser': False})):
                postgres.user_update(
                    'test_username',
                    user='test_user',
                    host='test_host',
                    port='test_port',
                    maintenance_db='test_maint',
                    password='test_pass',
                    createdb=False,
                    createroles=True,
                    createuser=False,
                    encrypted=False,
                    inherit=True,
                    login=True,
                    replication=False,
                    groups='test_groups',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    re.match(
                        'ALTER ROLE "test_username" WITH  INHERIT NOCREATEDB '
                        'CREATEROLE NOREPLICATION LOGIN;'
                        ' GRANT "test_groups" TO "test_username"',
                        postgres._run_psql.call_args[0][0][14]
                    )
                )

    def test_user_update3(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.role_get',
                       Mock(return_value={'superuser': False})):
                postgres.user_update(
                    'test_username',
                    user='test_user',
                    host='test_host',
                    port='test_port',
                    maintenance_db='test_maint',
                    password='test_pass',
                    createdb=False,
                    createroles=True,
                    createuser=False,
                    encrypted=False,
                    inherit=True,
                    login=True,
                    rolepassword=False,
                    replication=False,
                    groups='test_groups',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    re.match(
                        'ALTER ROLE "test_username" WITH  INHERIT NOCREATEDB '
                        'CREATEROLE NOREPLICATION LOGIN NOPASSWORD;'
                        ' GRANT "test_groups" TO "test_username"',
                        postgres._run_psql.call_args[0][0][14]
                    )
                )

    def test_user_update_encrypted_passwd(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.role_get',
                       Mock(return_value={'superuser': False})):
                postgres.user_update(
                    'test_username',
                    user='test_user',
                    host='test_host',
                    port='test_port',
                    maintenance_db='test_maint',
                    password='test_pass',
                    createdb=False,
                    createroles=True,
                    createuser=False,
                    encrypted=True,
                    inherit=True,
                    login=True,
                    rolepassword='foobar',
                    replication=False,
                    groups='test_groups',
                    runas='foo'
                )
                # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
                # The first 14 elements of this list are initial args used in all (or
                # virtually all) commands run through _run_psql(), so the actual SQL
                # query will be in the 15th argument.
                self.assertTrue(
                    re.match(
                        'ALTER ROLE "test_username" WITH  INHERIT NOCREATEDB '
                        'CREATEROLE NOREPLICATION LOGIN '
                        'ENCRYPTED PASSWORD '
                        '[\'"]{0,5}md531c27e68d3771c392b52102c01be1da1[\'"]{0,5}'
                        '; GRANT "test_groups" TO "test_username"',
                        postgres._run_psql.call_args[0][0][14]
                    )
                )

    def test_version(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0, 'stdout': '9.1.9'})):
            postgres.version(
                user='test_user',
                host='test_host',
                port='test_port',
                maintenance_db='test_maint',
                password='test_pass',
                runas='foo'
            )
            # postgres._run_psql.call_args[0][0] will contain the list of CLI args.
            # The first 14 elements of this list are initial args used in all (or
            # virtually all) commands run through _run_psql(), so the actual SQL
            # query will be in the 15th argument.
            self.assertTrue(
                re.match(
                    'SELECT setting FROM pg_catalog.pg_settings',
                    postgres._run_psql.call_args[0][0][14]
                )
            )

    def test_installed_extensions(self):
        with patch('salt.modules.postgres.psql_query',
                   Mock(return_value=[{'extname': "foo", 'extversion': "1"}])):
            exts = postgres.installed_extensions()
            self.assertEqual(
                exts,
                {'foo': {'extversion': '1', 'extname': 'foo'}}
            )

    def test_available_extensions(self):
        with patch('salt.modules.postgres.psql_query',
                   Mock(return_value=[{'name': "foo", 'default_version': "1"}])):
            exts = postgres.available_extensions()
            self.assertEqual(
                exts,
                {'foo': {'default_version': '1', 'name': 'foo'}}
            )

    def test_drop_extension2(self):
        with patch('salt.modules.postgres.installed_extensions',
                   Mock(side_effect=[{}, {}])):
            with patch('salt.modules.postgres._psql_prepare_and_run',
                       Mock(return_value=None)):
                with patch('salt.modules.postgres.available_extensions',
                           Mock(return_value={'foo': {'default_version': '1', 'name': 'foo'}})):
                    self.assertEqual(postgres.drop_extension('foo'), True)

    def test_drop_extension3(self):
        with patch('salt.modules.postgres.installed_extensions',
                   Mock(side_effect=[{'foo': {'extversion': '1', 'extname': 'foo'}}, {}])):
            with patch('salt.modules.postgres._psql_prepare_and_run',
                       Mock(return_value=None)):
                with patch('salt.modules.postgres.available_extensions',
                           Mock(return_value={'foo': {'default_version': '1', 'name': 'foo'}})):
                    self.assertEqual(postgres.drop_extension('foo'), True)

    def test_drop_extension1(self):
        with patch('salt.modules.postgres.installed_extensions',
                   Mock(side_effect=[{'foo': {'extversion': '1', 'extname': 'foo'}},
                                     {'foo': {'extversion': '1', 'extname': 'foo'}}])):
            with patch('salt.modules.postgres._psql_prepare_and_run',
                       Mock(return_value=None)):
                with patch('salt.modules.postgres.available_extensions',
                           Mock(return_value={'foo': {'default_version': '1', 'name': 'foo'}})):
                    self.assertEqual(postgres.drop_extension('foo'), False)

    def test_create_mtdata(self):
        with patch('salt.modules.postgres.installed_extensions',
                   Mock(return_value={
                       'foo': {'extversion': '0.8',
                               'extrelocatable': 't',
                               'schema_name': 'foo',
                               'extname': 'foo'}},
                       )):
            with patch('salt.modules.postgres.available_extensions',
                       Mock(return_value={'foo': {'default_version': '1.4',
                                                  'name': 'foo'}})):
                ret = postgres.create_metadata('foo', schema='bar', ext_version='1.4')
                self.assertTrue(postgres._EXTENSION_INSTALLED in ret)
                self.assertTrue(postgres._EXTENSION_TO_UPGRADE in ret)
                self.assertTrue(postgres._EXTENSION_TO_MOVE in ret)
                ret = postgres.create_metadata('foo', schema='foo', ext_version='0.4')
                self.assertTrue(postgres._EXTENSION_INSTALLED in ret)
                self.assertFalse(postgres._EXTENSION_TO_UPGRADE in ret)
                self.assertFalse(postgres._EXTENSION_TO_MOVE in ret)
                ret = postgres.create_metadata('foo')
                self.assertTrue(postgres._EXTENSION_INSTALLED in ret)
                self.assertFalse(postgres._EXTENSION_TO_UPGRADE in ret)
                self.assertFalse(postgres._EXTENSION_TO_MOVE in ret)
                ret = postgres.create_metadata('foobar')
                self.assertTrue(postgres._EXTENSION_NOT_INSTALLED in ret)
                self.assertFalse(postgres._EXTENSION_INSTALLED in ret)
                self.assertFalse(postgres._EXTENSION_TO_UPGRADE in ret)
                self.assertFalse(postgres._EXTENSION_TO_MOVE in ret)

    def test_create_extension_newerthan(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch('salt.modules.postgres.create_metadata',
                   Mock(side_effect=[
                       # create succeeded
                       [postgres._EXTENSION_NOT_INSTALLED],
                       [postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_NOT_INSTALLED],
                       [postgres._EXTENSION_INSTALLED],
                       # create failed
                       [postgres._EXTENSION_NOT_INSTALLED],
                       [postgres._EXTENSION_NOT_INSTALLED],
                       # move+upgrade succeeded
                       [postgres._EXTENSION_TO_MOVE,
                        postgres._EXTENSION_TO_UPGRADE,
                        postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_INSTALLED],
                       # move succeeded
                       [postgres._EXTENSION_TO_MOVE,
                        postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_INSTALLED],
                       # upgrade succeeded
                       [postgres._EXTENSION_TO_UPGRADE,
                        postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_INSTALLED],
                       # upgrade failed
                       [postgres._EXTENSION_TO_UPGRADE, postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_TO_UPGRADE, postgres._EXTENSION_INSTALLED],
                       # move failed
                       [postgres._EXTENSION_TO_MOVE, postgres._EXTENSION_INSTALLED],
                       [postgres._EXTENSION_TO_MOVE, postgres._EXTENSION_INSTALLED]])):
            with patch('salt.modules.postgres._psql_prepare_and_run',
                       Mock(return_value=None)):
                with patch('salt.modules.postgres.available_extensions',
                           Mock(return_value={'foo': {'default_version': '1.4',
                                                      'name': 'foo'}})):
                    self.assertTrue(postgres.create_extension('foo'))
                    self.assertTrue(re.match(
                        'CREATE EXTENSION IF NOT EXISTS "foo" ;',
                        postgres._psql_prepare_and_run.call_args[0][0][1]))
                    self.assertTrue(postgres.create_extension(
                        'foo', schema='a', ext_version='b', from_version='c'))
                    self.assertTrue(re.match(
                        'CREATE EXTENSION IF NOT EXISTS "foo" '
                        'WITH SCHEMA "a" VERSION b FROM c ;',
                        postgres._psql_prepare_and_run.call_args[0][0][1]))
                    self.assertFalse(postgres.create_extension('foo'))
                    ret = postgres.create_extension('foo', ext_version='a', schema='b')
                    self.assertTrue(ret)
                    self.assertTrue(re.match(
                        'ALTER EXTENSION "foo" SET SCHEMA "b";'
                        ' ALTER EXTENSION "foo" UPDATE TO a;',
                        postgres._psql_prepare_and_run.call_args[0][0][1]))
                    ret = postgres.create_extension('foo', ext_version='a', schema='b')
                    self.assertTrue(ret)
                    self.assertTrue(re.match(
                        'ALTER EXTENSION "foo" SET SCHEMA "b";',
                        postgres._psql_prepare_and_run.call_args[0][0][1]))
                    ret = postgres.create_extension('foo', ext_version='a', schema='b')
                    self.assertTrue(ret)
                    self.assertTrue(re.match(
                        'ALTER EXTENSION "foo" UPDATE TO a;',
                        postgres._psql_prepare_and_run.call_args[0][0][1]))
                    self.assertFalse(postgres.create_extension(
                        'foo', ext_version='a', schema='b'))
                    self.assertFalse(postgres.create_extension(
                        'foo', ext_version='a', schema='b'))

    def test_encrypt_passwords(self):
        self.assertEqual(
            postgres._maybe_encrypt_password(
                'foo', 'bar', False),
            'bar')
        self.assertEqual(
            postgres._maybe_encrypt_password(
                'foo', 'bar', True),
            'md596948aad3fcae80c08a35c9b5958cd89')

    def test_schema_list(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_list_schema_csv})):
            ret = postgres.schema_list(
                'maint_db',
                db_user='testuser',
                db_host='testhost',
                db_port='testport',
                db_password='foo'
            )
            self.assertDictEqual(ret, {
                'public': {'acl': '{postgres=UC/postgres,=UC/postgres}',
                           'owner': 'postgres'},
                'pg_toast': {'acl': '', 'owner': 'postgres'}
                })

    def test_schema_exists(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.psql_query',
                       Mock(return_value=[
                           {
                               'name': 'public',
                               'acl': '{postgres=UC/postgres,=UC/postgres}',
                               'owner': 'postgres'
                           }])):
                ret = postgres.schema_exists(
                    'template1',
                    'public'
                )
                self.assertTrue(ret)

    def test_schema_get(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.psql_query',
                       Mock(return_value=[
                           {
                               'name': 'public',
                               'acl': '{postgres=UC/postgres,=UC/postgres}',
                               'owner': 'postgres'
                           }])):
                ret = postgres.schema_get(
                    'template1',
                    'public'
                )
                self.assertTrue(ret)

    def test_schema_get_again(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.psql_query',
                       Mock(return_value=[
                           {
                               'name': 'public',
                               'acl': '{postgres=UC/postgres,=UC/postgres}',
                               'owner': 'postgres'
                           }])):
                ret = postgres.schema_get(
                    'template1',
                    'pg_toast'
                )
                self.assertFalse(ret)

    def test_schema_create(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.schema_exists', Mock(return_value=False)):
                postgres.schema_create(
                    'maint_db',
                    'testschema',
                    user='user',
                    db_host='testhost',
                    db_port='testport',
                    db_user='testuser',
                    db_password='testpassword'
                )
                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'maint_db',
                     '-c', 'CREATE SCHEMA "testschema"'],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_schema_create2(self):
        with patch('salt.modules.postgres.schema_exists', Mock(return_value=True)):
            ret = postgres.schema_create('test_db',
                                         'test_schema',
                                         user='user',
                                         db_host='test_host',
                                         db_port='test_port',
                                         db_user='test_user',
                                         db_password='test_password'
                                         )
            self.assertFalse(ret)

    def test_schema_remove(self):
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.schema_exists', Mock(return_value=True)):
                postgres.schema_remove(
                    'maint_db',
                    'testschema',
                    user='user',
                    db_host='testhost',
                    db_port='testport',
                    db_user='testuser',
                    db_password='testpassword'
                )
                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'maint_db',
                     '-c', 'DROP SCHEMA "testschema"'],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_schema_remove2(self):
        with patch('salt.modules.postgres.schema_exists', Mock(return_value=False)):
            ret = postgres.schema_remove('test_db',
                                         'test_schema',
                                         user='user',
                                         db_host='test_host',
                                         db_port='test_port',
                                         db_user='test_user',
                                         db_password='test_password'
                                         )
            self.assertFalse(ret)

    def test_language_list(self):
        '''
        Test language listing
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_list_language_csv})):
            ret = postgres.language_list(
                'testdb',
                user='testuser',
                host='testhost',
                port='testport',
                password='foo'
            )
            self.assertDictEqual(ret,
                {'c': 'c',
                'internal': 'internal',
                'plpgsql': 'plpgsql',
                'sql': 'sql'})

    def test_language_exists(self):
        '''
        Test language existence check
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.psql_query',
                       Mock(return_value=[
                           {'Name': 'internal'},
                           {'Name': 'c'},
                           {'Name': 'sql'},
                           {'Name': 'plpgsql'}])):
                with patch('salt.modules.postgres.language_exists',
                           Mock(return_value=True)):
                    ret = postgres.language_exists(
                        'sql',
                        'testdb'
                    )
                    self.assertTrue(ret)

    def test_language_create(self):
        '''
        Test language creation - does not exist in db
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.language_exists', Mock(return_value=False)):
                postgres.language_create(
                    'plpythonu',
                    'testdb',
                    runas='user',
                    host='testhost',
                    port='testport',
                    user='testuser',
                    password='testpassword'
                )
                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'testdb',
                     '-c', 'CREATE LANGUAGE plpythonu'],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_language_create_exists(self):
        '''
        Test language creation - already exists in db
        '''
        with patch('salt.modules.postgres.language_exists', Mock(return_value=True)):
            ret = postgres.language_create(
                'plpythonu',
                'testdb',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )
            self.assertFalse(ret)

    def test_language_remove(self):
        '''
        Test language removal - exists in db
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.language_exists',
                       Mock(return_value=True)):
                postgres.language_remove(
                    'plpgsql',
                    'testdb',
                    runas='user',
                    host='testhost',
                    port='testport',
                    user='testuser',
                    password='testpassword'
                )
                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'testdb',
                     '-c', 'DROP LANGUAGE plpgsql'],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_language_remove_non_exist(self):
        '''
        Test language removal - does not exist in db
        '''
        with patch('salt.modules.postgres.language_exists',
                   Mock(return_value=False)):
            ret = postgres.language_remove(
                'plpgsql',
                'testdb',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )
            self.assertFalse(ret)

    def test_privileges_list_table(self):
        '''
        Test privilege listing on a table
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_privileges_list_table_csv})):
            ret = postgres.privileges_list(
                'awl',
                'table',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )
            expected = {
                "bayestest": {
                    "INSERT": False,
                    "UPDATE": False,
                    "SELECT": False,
                    "DELETE": False,
                },
                "baruwa": {
                    "INSERT": True,
                    "TRUNCATE": True,
                    "UPDATE": True,
                    "TRIGGER": True,
                    "REFERENCES": True,
                    "SELECT": True,
                    "DELETE": True,
                },
                "baruwatest": {
                    "INSERT": False,
                    "TRUNCATE": False,
                    "UPDATE": False,
                    "TRIGGER": False,
                    "REFERENCES": False,
                    "SELECT": False,
                    "DELETE": False,
                },
            }

            self.assertDictEqual(ret, expected)

            query = ("COPY (SELECT relacl AS name FROM pg_catalog.pg_class c "
            "JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace "
            "WHERE nspname = 'public' AND relname = 'awl' AND relkind = 'r' "
            "ORDER BY relname) TO STDOUT WITH CSV HEADER")

            postgres._run_psql.assert_called_once_with(
                ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                 '--no-password', '--username', 'testuser', '--host',
                 'testhost', '--port', 'testport', '--dbname', 'db_name',
                 '-v', 'datestyle=ISO,MDY', '-c', query],
                host='testhost', port='testport',
                password='testpassword', user='testuser', runas='user')

    def test_privileges_list_group(self):
        '''
        Test privilege listing on a group
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_privileges_list_group_csv})):
            ret = postgres.privileges_list(
                'admin',
                'group',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )
            expected = {
                'baruwa': False,
                'baruwatest': False,
                'baruwatest2': True,
            }

            self.assertDictEqual(ret, expected)

            query = ("COPY (SELECT rolname, admin_option "
            "FROM pg_catalog.pg_auth_members m JOIN pg_catalog.pg_roles r "
            "ON m.member=r.oid WHERE m.roleid IN (SELECT oid FROM "
            "pg_catalog.pg_roles WHERE rolname='admin') ORDER BY rolname) "
            "TO STDOUT WITH CSV HEADER")

            postgres._run_psql.assert_called_once_with(
                ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                 '--no-password', '--username', 'testuser', '--host',
                 'testhost', '--port', 'testport', '--dbname', 'db_name',
                 '-v', 'datestyle=ISO,MDY', '-c', query],
                host='testhost', port='testport',
                password='testpassword', user='testuser', runas='user')

    def test_has_privileges_on_table(self):
        '''
        Test privilege checks on table
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_privileges_list_table_csv})):
            ret = postgres.has_privileges(
                'baruwa',
                'awl',
                'table',
                'SELECT,INSERT',
                grant_option=True,
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertTrue(ret)

            ret = postgres.has_privileges(
                'baruwa',
                'awl',
                'table',
                'ALL',
                grant_option=True,
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertTrue(ret)

            ret = postgres.has_privileges(
                'bayestest',
                'awl',
                'table',
                'SELECT,INSERT,TRUNCATE',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertFalse(ret)

            ret = postgres.has_privileges(
                'bayestest',
                'awl',
                'table',
                'SELECT,INSERT',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertTrue(ret)

    def test_has_privileges_on_group(self):
        '''
        Test privilege checks on group
        '''
        with patch('salt.modules.postgres._run_psql',
                   Mock(return_value={'retcode': 0,
                                      'stdout': test_privileges_list_group_csv})):
            ret = postgres.has_privileges(
                'baruwa',
                'admin',
                'group',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertTrue(ret)

            ret = postgres.has_privileges(
                'baruwa',
                'admin',
                'group',
                grant_option=True,
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertFalse(ret)

            ret = postgres.has_privileges(
                'tony',
                'admin',
                'group',
                maintenance_db='db_name',
                runas='user',
                host='testhost',
                port='testport',
                user='testuser',
                password='testpassword'
            )

            self.assertFalse(ret)

    def test_privileges_grant_table(self):
        '''
        Test granting privileges on table
        '''
        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=False)):
                ret = postgres.privileges_grant(
                   'baruwa',
                   'awl',
                   'table',
                   'ALL',
                   grant_option=True,
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'GRANT ALL ON TABLE public."awl" TO "baruwa" WITH GRANT OPTION'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=False)):
                ret = postgres.privileges_grant(
                   'baruwa',
                   'awl',
                   'table',
                   'ALL',
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'GRANT ALL ON TABLE public."awl" TO "baruwa"'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

        # Test grant on all tables
        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=False)):
                ret = postgres.privileges_grant(
                   'baruwa',
                   'ALL',
                   'table',
                   'SELECT',
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'GRANT SELECT ON ALL TABLES IN SCHEMA public TO "baruwa"'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_privileges_grant_group(self):
        '''
        Test granting privileges on group
        '''
        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=False)):
                ret = postgres.privileges_grant(
                   'baruwa',
                   'admins',
                   'group',
                   grant_option=True,
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'GRANT admins TO "baruwa" WITH ADMIN OPTION'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=False)):
                ret = postgres.privileges_grant(
                   'baruwa',
                   'admins',
                   'group',
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'GRANT admins TO "baruwa"'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_privileges_revoke_table(self):
        '''
        Test revoking privileges on table
        '''
        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=True)):
                ret = postgres.privileges_revoke(
                   'baruwa',
                   'awl',
                   'table',
                   'ALL',
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'REVOKE ALL ON TABLE public.awl FROM baruwa'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_privileges_revoke_group(self):
        '''
        Test revoking privileges on group
        '''
        with patch('salt.modules.postgres._run_psql',
            Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.has_privileges',
                    Mock(return_value=True)):
                ret = postgres.privileges_revoke(
                   'baruwa',
                   'admins',
                   'group',
                   maintenance_db='db_name',
                   runas='user',
                   host='testhost',
                   port='testport',
                   user='testuser',
                   password='testpassword'
                )

                query = 'REVOKE admins FROM baruwa'

                postgres._run_psql.assert_called_once_with(
                    ['/usr/bin/pgsql', '--no-align', '--no-readline', '--no-psqlrc',
                     '--no-password', '--username', 'testuser', '--host',
                     'testhost', '--port', 'testport', '--dbname', 'db_name',
                     '-c', query],
                    host='testhost', port='testport',
                    password='testpassword', user='testuser', runas='user')

    def test_datadir_init(self):
        '''
        Test Initializing a postgres data directory
        '''
        with patch('salt.modules.postgres._run_initdb',
                   Mock(return_value={'retcode': 0})):
            with patch('salt.modules.postgres.datadir_exists',
                       Mock(return_value=False)):
                name = '/var/lib/pgsql/data'
                ret = postgres.datadir_init(
                        name,
                        user='postgres',
                        password='test',
                        runas='postgres')
                postgres._run_initdb.assert_called_once_with(
                    name,
                    auth='password',
                    encoding='UTF8',
                    locale=None,
                    password='test',
                    runas='postgres',
                    user='postgres',
                )
                self.assertTrue(ret)

    def test_datadir_exists(self):
        '''
        Test Checks if postgres data directory has been initialized
        '''
        with patch('os.path.isfile', Mock(return_value=True)):
            name = '/var/lib/pgsql/data'
            ret = postgres.datadir_exists(name)
            self.assertTrue(ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
import tests.support.mock as mock

import salt.config
import salt.syspaths


class ConfigTest(TestCase):

    def test_validate_bad_pillar_roots(self):
        expected = salt.config._expand_glob_path(
            [salt.syspaths.BASE_PILLAR_ROOTS_DIR]
        )
        with mock.patch('salt.config._normalize_roots') as mk:
            ret = salt.config._validate_pillar_roots(None)
            assert not mk.called
        assert ret == {'base': expected}

    def test_validate_bad_file_roots(self):
        expected = salt.config._expand_glob_path(
            [salt.syspaths.BASE_FILE_ROOTS_DIR]
        )
        with mock.patch('salt.config._normalize_roots') as mk:
            ret = salt.config._validate_file_roots(None)
            assert not mk.called
        assert ret == {'base': expected}



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.keyboard as keyboard


@skipIf(NO_MOCK, NO_MOCK_REASON)
class KeyboardTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.keyboard
    '''
    def setup_loader_modules(self):
        return {keyboard: {}}

    # 'get_sys' function tests: 1

    def test_get_sys(self):
        '''
        Test if it get current system keyboard setting
        '''
        mock = MagicMock(return_value='X11 Layout=us')
        with patch.dict(keyboard.__grains__, {'os_family': 'RedHat'}):
            with patch.dict(keyboard.__salt__, {'cmd.run': mock}):
                self.assertEqual(keyboard.get_sys(), 'us')

    # 'set_sys' function tests: 1

    def test_set_sys(self):
        '''
        Test if it set current system keyboard setting
        '''
        mock = MagicMock(return_value='us')
        with patch.dict(keyboard.__grains__, {'os_family': 'RedHat'}):
            with patch.dict(keyboard.__salt__, {'cmd.run': mock}):
                with patch.dict(keyboard.__salt__, {'file.sed': MagicMock()}):
                    self.assertEqual(keyboard.set_sys('us'), 'us')

    # 'get_x' function tests: 1

    def test_get_x(self):
        '''
        Test if it get current X keyboard setting
        '''
        mock = MagicMock(return_value='layout:     us')
        with patch.dict(keyboard.__salt__, {'cmd.run': mock}):
            self.assertEqual(keyboard.get_x(), 'us')

    # 'set_x' function tests: 1

    def test_set_x(self):
        '''
        Test if it set current X keyboard setting
        '''
        mock = MagicMock(return_value='us')
        with patch.dict(keyboard.__salt__, {'cmd.run': mock}):
            self.assertEqual(keyboard.set_x('us'), 'us')



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)
import salt.modules.libcloud_compute as libcloud_compute

from libcloud.compute.base import (
    BaseDriver, Node,
    NodeSize, NodeState, NodeLocation,
    StorageVolume, StorageVolumeState,
    VolumeSnapshot, NodeImage, KeyPair)


class MockComputeDriver(BaseDriver):
    def __init__(self):
        self._TEST_SIZE = NodeSize(
            id='test_id', name='test_size',
            ram=4096, disk=10240, bandwidth=100000, price=0,
            driver=self)
        self._TEST_NODE = Node(
            id='test_id', name='test_node',
            state=NodeState.RUNNING, public_ips=['1.2.3.4'],
            private_ips=['2.3.4.5'], driver=self,
            size=self._TEST_SIZE, extra={
                'ex_key': 'ex_value'
            })
        self._TEST_LOCATION = NodeLocation(
            id='test_location',
            name='location1',
            country='Australia',
            driver=self)
        self._TEST_VOLUME = StorageVolume(
            id='vol1',
            name='vol_name',
            size=40960,
            driver=self,
            state=StorageVolumeState.AVAILABLE,
            extra={
                'ex_key': 'ex_value'
            }
        )
        self._TEST_VOLUME_SNAPSHOT = VolumeSnapshot(
            id='snap1',
            size=80960,
            driver=self
        )
        self._TEST_IMAGE = NodeImage(
            id='image1',
            name='test_image',
            extra={
                'ex_key': 'ex_value'
            },
            driver=self
        )
        self._TEST_KEY_PAIR = KeyPair(
            name='test_key',
            fingerprint='abc123',
            public_key='pub123',
            private_key='priv123',
            driver=self,
            extra={
                'ex_key': 'ex_value'
            }
        )

    def list_nodes(self):
        return [self._TEST_NODE]

    def list_sizes(self, location=None):
        if location:
            assert location.id == 'test_location'
        return [self._TEST_SIZE]

    def list_locations(self):
        return [self._TEST_LOCATION]

    def reboot_node(self, node):
        assert node.id == 'test_id'
        return True

    def destroy_node(self, node):
        assert node.id == 'test_id'
        return True

    def list_volumes(self):
        return [self._TEST_VOLUME]

    def list_volume_snapshots(self, volume):
        assert volume.id == 'vol1'
        return [self._TEST_VOLUME_SNAPSHOT]

    def create_volume(self, size, name, location=None, snapshot=None):
        assert size == 9000
        assert name == 'test_new_volume'
        if location:
            assert location.country == 'Australia'
        return self._TEST_VOLUME

    def create_volume_snapshot(self, volume, name=None):
        assert volume.id == 'vol1'
        if name:
            assert name == 'test_snapshot'
        return self._TEST_VOLUME_SNAPSHOT

    def attach_volume(self, node, volume, device=None):
        assert node.id == 'test_id'
        assert volume.id == 'vol1'
        if device:
            assert device == '/dev/sdc'
        return True

    def detach_volume(self, volume):
        assert volume.id == 'vol1'
        return True

    def destroy_volume(self, volume):
        assert volume.id == 'vol1'
        return True

    def destroy_volume_snapshot(self, snapshot):
        assert snapshot.id == 'snap1'
        return True

    def list_images(self, location=None):
        if location:
            assert location.id == 'test_location'
        return [self._TEST_IMAGE]

    def create_image(self, node, name, description=None):
        assert node.id == 'test_id'
        return self._TEST_IMAGE

    def delete_image(self, node_image):
        return True

    def get_image(self, image_id):
        assert image_id == 'image1'
        return self._TEST_IMAGE

    def copy_image(self, source_region, node_image, name, description=None):
        assert source_region == 'us-east1'
        assert node_image.id == 'image1'
        assert name == 'copy_test'
        return self._TEST_IMAGE

    def list_key_pairs(self):
        return [self._TEST_KEY_PAIR]

    def get_key_pair(self, name):
        assert name == 'test_key'
        return self._TEST_KEY_PAIR

    def create_key_pair(self, name):
        assert name == 'test_key'
        return self._TEST_KEY_PAIR

    def import_key_pair_from_string(self, name, key_material):
        assert name == 'test_key'
        assert key_material == 'test_key_value'
        return self._TEST_KEY_PAIR

    def import_key_pair_from_file(self, name, key_file_path):
        assert name == 'test_key'
        assert key_file_path == '/path/to/key'
        return self._TEST_KEY_PAIR

    def delete_key_pair(self, key_pair):
        assert key_pair.name == 'test_key'
        return True


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.modules.libcloud_compute._get_driver',
       MagicMock(return_value=MockComputeDriver()))
class LibcloudComputeModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                })
            }
        }
        if libcloud_compute.HAS_LIBCLOUD is False:
            module_globals['sys.modules'] = {'libcloud': MagicMock()}

        return {libcloud_compute: module_globals}

    def test_module_creation(self):
        client = libcloud_compute._get_driver('test')
        self.assertFalse(client is None)

    def test_init(self):
        with patch('salt.utils.compat.pack_dunder', return_value=False) as dunder:
            libcloud_compute.__init__(None)
            dunder.assert_called_with('salt.modules.libcloud_compute')

    def _validate_node(self, node):
        self.assertEqual(node['name'], 'test_node')
        self.assertEqual(node['id'], 'test_id')
        self.assertEqual(node['private_ips'], ['2.3.4.5'])
        self.assertEqual(node['public_ips'], ['1.2.3.4'])
        self.assertEqual(node['size']['name'], 'test_size')

    def _validate_size(self, size):
        self.assertEqual(size['id'], 'test_id')
        self.assertEqual(size['name'], 'test_size')
        self.assertEqual(size['ram'], 4096)

    def _validate_location(self, location):
        self.assertEqual(location['id'], 'test_location')
        self.assertEqual(location['name'], 'location1')
        self.assertEqual(location['country'], 'Australia')

    def _validate_volume(self, volume):
        self.assertEqual(volume['id'], 'vol1')
        self.assertEqual(volume['name'], 'vol_name')
        self.assertEqual(volume['size'], 40960)
        self.assertEqual(volume['state'], 'available')
        self.assertEqual(volume['extra'], {'ex_key': 'ex_value'})

    def _validate_volume_snapshot(self, volume):
        self.assertEqual(volume['id'], 'snap1')
        self.assertEqual(volume['size'], 80960)

    def _validate_image(self, image):
        self.assertEqual(image['id'], 'image1')
        self.assertEqual(image['name'], 'test_image')
        self.assertEqual(image['extra'], {'ex_key': 'ex_value'})

    def _validate_key_pair(self, key):
        self.assertEqual(key['name'], 'test_key')
        self.assertEqual(key['fingerprint'], 'abc123')
        self.assertEqual(key['extra'], {'ex_key': 'ex_value'})

    def test_list_nodes(self):
        nodes = libcloud_compute.list_nodes('test')
        self.assertEqual(len(nodes), 1)
        self._validate_node(nodes[0])

    def test_list_sizes(self):
        sizes = libcloud_compute.list_sizes('test')
        self.assertEqual(len(sizes), 1)
        self._validate_size(sizes[0])

    def test_list_sizes_location(self):
        sizes = libcloud_compute.list_sizes('test', location_id='test_location')
        self.assertEqual(len(sizes), 1)
        self._validate_size(sizes[0])

    def test_list_locations(self):
        locations = libcloud_compute.list_locations('test')
        self.assertEqual(len(locations), 1)
        self._validate_location(locations[0])

    def test_reboot_node(self):
        result = libcloud_compute.reboot_node('test_id', 'test')
        self.assertTrue(result)

    def test_reboot_node_invalid(self):
        with self.assertRaises(ValueError):
            libcloud_compute.reboot_node('foo_node', 'test')

    def test_destroy_node(self):
        result = libcloud_compute.destroy_node('test_id', 'test')
        self.assertTrue(result)

    def test_destroy_node_invalid(self):
        with self.assertRaises(ValueError):
            libcloud_compute.destroy_node('foo_node', 'test')

    def test_list_volumes(self):
        volumes = libcloud_compute.list_volumes('test')
        self.assertEqual(len(volumes), 1)
        self._validate_volume(volumes[0])

    def test_list_volume_snapshots(self):
        volumes = libcloud_compute.list_volume_snapshots('vol1', 'test')
        self.assertEqual(len(volumes), 1)
        self._validate_volume_snapshot(volumes[0])

    def test_create_volume(self):
        volume = libcloud_compute.create_volume(9000, 'test_new_volume', 'test')
        self._validate_volume(volume)

    def test_create_volume_in_location(self):
        volume = libcloud_compute.create_volume(9000, 'test_new_volume', 'test', location_id='test_location')
        self._validate_volume(volume)

    def test_create_volume_snapshot(self):
        snapshot = libcloud_compute.create_volume_snapshot('vol1', 'test')
        self._validate_volume_snapshot(snapshot)

    def test_create_volume_snapshot_named(self):
        snapshot = libcloud_compute.create_volume_snapshot('vol1', 'test', name='test_snapshot')
        self._validate_volume_snapshot(snapshot)

    def test_attach_volume(self):
        result = libcloud_compute.attach_volume('test_id', 'vol1', 'test')
        self.assertTrue(result)

    def test_detatch_volume(self):
        result = libcloud_compute.detach_volume('vol1', 'test')
        self.assertTrue(result)

    def test_destroy_volume(self):
        result = libcloud_compute.destroy_volume('vol1', 'test')
        self.assertTrue(result)

    def test_destroy_volume_snapshot(self):
        result = libcloud_compute.destroy_volume_snapshot('vol1', 'snap1', 'test')
        self.assertTrue(result)

    def test_list_images(self):
        images = libcloud_compute.list_images('test')
        self.assertEqual(len(images), 1)
        self._validate_image(images[0])

    def test_list_images_in_location(self):
        images = libcloud_compute.list_images('test', location_id='test_location')
        self.assertEqual(len(images), 1)
        self._validate_image(images[0])

    def test_create_image(self):
        image = libcloud_compute.create_image('test_id', 'new_image', 'test')
        self._validate_image(image)

    def test_delete_image(self):
        result = libcloud_compute.delete_image('image1', 'test')
        self.assertTrue(result)

    def test_get_image(self):
        image = libcloud_compute.get_image('image1', 'test')
        self._validate_image(image)

    def test_copy_image(self):
        new_image = libcloud_compute.copy_image('us-east1', 'image1', 'copy_test', 'test')
        self._validate_image(new_image)

    def test_list_key_pairs(self):
        keys = libcloud_compute.list_key_pairs('test')
        self.assertEqual(len(keys), 1)
        self._validate_key_pair(keys[0])

    def test_get_key_pair(self):
        key = libcloud_compute.get_key_pair('test_key', 'test')
        self._validate_key_pair(key)

    def test_create_key_pair(self):
        key = libcloud_compute.create_key_pair('test_key', 'test')
        self._validate_key_pair(key)

    def test_import_key_string(self):
        key = libcloud_compute.import_key_pair('test_key', 'test_key_value', 'test')
        self._validate_key_pair(key)

    def test_import_key_file(self):
        key = libcloud_compute.import_key_pair('test_key', '/path/to/key', 'test', key_type='FILE')
        self._validate_key_pair(key)

    def test_delete_key_pair(self):
        result = libcloud_compute.delete_key_pair('test_key', 'test')
        self.assertTrue(result)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

# Import Salt libs
import salt.renderers.nacl as nacl


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NaclTestCase(TestCase, LoaderModuleMockMixin):
    '''
    unit test NaCl renderer
    '''
    def setup_loader_modules(self):
        return {nacl: {}}

    def test__decrypt_object(self):
        '''
        test _decrypt_object
        '''
        secret = 'Use more salt.'
        crypted = 'NACL[MRN3cc+fmdxyQbz6WMF+jq1hKdU5X5BBI7OjK+atvHo1ll+w1gZ7XyWtZVfq9gK9rQaMfkDxmidJKwE0Mw==]'

        secret_map = {'secret': secret}
        crypted_map = {'secret': crypted}

        secret_list = [secret]
        crypted_list = [crypted]

        with patch.dict(nacl.__salt__, {'nacl.dec': MagicMock(return_value=secret)}):
            self.assertEqual(nacl._decrypt_object(secret), secret)
            self.assertEqual(nacl._decrypt_object(crypted), secret)
            self.assertEqual(nacl._decrypt_object(crypted_map), secret_map)
            self.assertEqual(nacl._decrypt_object(crypted_list), secret_list)
            self.assertEqual(nacl._decrypt_object(None), None)

    def test_render(self):
        '''
        test render
        '''
        secret = 'Use more salt.'
        crypted = 'NACL[MRN3cc+fmdxyQbz6WMF+jq1hKdU5X5BBI7OjK+atvHo1ll+w1gZ7XyWtZVfq9gK9rQaMfkDxmidJKwE0Mw==]'
        with patch.dict(nacl.__salt__, {'nacl.dec': MagicMock(return_value=secret)}):
            self.assertEqual(nacl.render(crypted), secret)



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, mock_open
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.log as log

import logging
_log = logging.getLogger(__name__)


_STUB_LOG_ENTRY = 'Jun 29 12:58:51 hostname sshd[6536]: ' \
                  'pam_unix(sshd:session): session opened ' \
                  'for user username by (uid=0)\n'


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LogBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.log
    '''

    def setup_loader_modules(self):
        return {
            log: {
                '__context__': {'log.loc': 2},
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = log.validate(config)

        self.assertEqual(ret, (False, 'Configuration for log beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = log.validate(config)

        self.assertEqual(ret, (False, 'Configuration for log beacon '
                                      'must contain file option.'))

    def test_log_match(self):
        with patch('salt.utils.files.fopen',
                   mock_open(read_data=_STUB_LOG_ENTRY)):
            config = [{'file': '/var/log/auth.log',
                       'tags': {'sshd': {'regex': '.*sshd.*'}}
                       }]

            ret = log.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            _expected_return = [{'error': '',
                                 'match': 'yes',
                                 'raw': _STUB_LOG_ENTRY.rstrip('\n'),
                                 'tag': 'sshd'
                                 }]
            ret = log.beacon(config)
            self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-
'''
Tests for the Reg State
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, generate_random_name

# Import Salt libs
import salt.utils.platform
import salt.utils.win_reg as reg

log = logging.getLogger(__name__)

__testcontext__ = {}

UNICODE_VALUE_NAME = 'Unicode Key \N{TRADE MARK SIGN}'
UNICODE_VALUE = 'Unicode Value ' \
                '\N{COPYRIGHT SIGN},\N{TRADE MARK SIGN},\N{REGISTERED SIGN}'
FAKE_KEY = 'SOFTWARE\\{0}'.format(generate_random_name('SaltTesting-'))


@destructiveTest
@skipIf(not salt.utils.platform.is_windows(), 'Windows Specific Test')
class RegTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Reg state module tests
    These tests are destructive as the modify the registry
    '''
    def tearDown(self):
        reg.delete_key_recursive(hive='HKLM',
                                 key=FAKE_KEY)
        reg.delete_key_recursive(hive='HKLM',
                                 key=FAKE_KEY,
                                 use_32bit_registry=True)

    def test_present_reg_sz(self):
        '''
        Testing reg.present with REG_SZ
        '''
        log.debug('Testing reg.present with REG_SZ')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname='test_reg_sz',
                             vdata='fake string data')
        expected = {
            'reg': {
                'Added': {
                    'Entry': 'test_reg_sz',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': 'fake string data'}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM', key=FAKE_KEY, vname='test_reg_sz')
        expected = {
            'vtype': 'REG_SZ',
            'vname': 'test_reg_sz',
            'success': True,
            'hive': 'HKLM',
            'vdata': 'fake string data',
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_reg_sz_unicode_value(self):
        '''
        Testing reg.present with REG_SZ and a unicode value
        '''
        log.debug('Testing reg.present with REG_SZ and a unicode value')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname='test_reg_sz',
                             vdata=UNICODE_VALUE)
        expected = {
            'reg': {
                'Added': {
                    'Entry': 'test_reg_sz',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': UNICODE_VALUE}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM', key=FAKE_KEY, vname='test_reg_sz')
        expected = {
            'vtype': 'REG_SZ',
            'vname': 'test_reg_sz',
            'success': True,
            'hive': 'HKLM',
            'vdata': UNICODE_VALUE,
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_reg_sz_unicode_default_value(self):
        '''
        Testing reg.present with REG_SZ and a unicode default value
        '''
        log.debug('Testing reg.present with REG_SZ and a unicode default value')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vdata=UNICODE_VALUE)
        expected = {
            'reg': {
                'Added': {
                    'Entry': '(Default)',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': UNICODE_VALUE}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM', key=FAKE_KEY)

        expected = {
            'vtype': 'REG_SZ',
            'vname': '(Default)',
            'success': True,
            'hive': 'HKLM',
            'vdata': UNICODE_VALUE,
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_reg_sz_unicode_value_name(self):
        '''
        Testing reg.present with REG_SZ and a unicode value name
        '''
        log.debug('Testing reg.present with REG_SZ and a unicode value name')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname=UNICODE_VALUE_NAME,
                             vdata='fake string data')
        expected = {
            'reg': {
                'Added': {
                    'Entry': UNICODE_VALUE_NAME,
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': 'fake string data'}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM', key=FAKE_KEY, vname=UNICODE_VALUE_NAME)

        expected = {
            'vtype': 'REG_SZ',
            'vname': UNICODE_VALUE_NAME,
            'success': True,
            'hive': 'HKLM',
            'vdata': 'fake string data',
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_reg_binary(self):
        '''
        Testing reg.present with REG_BINARY
        '''
        test_data = 'Salty Test'
        log.debug('Testing reg.present with REG_BINARY')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname='test_reg_binary',
                             vtype='REG_BINARY',
                             vdata=test_data)
        expected = {
            'reg': {
                'Added': {
                    'Entry': 'test_reg_binary',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': test_data}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM', key=FAKE_KEY, vname='test_reg_binary')
        expected = {
            'vtype': 'REG_BINARY',
            'vname': 'test_reg_binary',
            'success': True,
            'hive': 'HKLM',
            'vdata': test_data.encode('utf-8'),
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_reg_multi_sz(self):
        '''
        Testing reg.present with REG_MULTI_SZ
        '''
        log.debug('Testing reg.present with REG_MULTI_SZ')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname='test_reg_multi_sz',
                             vtype='REG_MULTI_SZ',
                             vdata=['item1', 'item2'])
        expected = {
            'reg': {
                'Added': {
                    'Entry': 'test_reg_multi_sz',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': ['item1', 'item2']}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM',
                             key=FAKE_KEY,
                             vname='test_reg_multi_sz')
        expected = {
            'vtype': 'REG_MULTI_SZ',
            'vname': 'test_reg_multi_sz',
            'success': True,
            'hive': 'HKLM',
            'vdata': ['item1', 'item2'],
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)

    def test_present_32_bit(self):
        '''
        Testing reg.present with REG_SZ using 32bit registry
        '''
        log.debug('Testing reg.present with REG_SZ using 32bit registry')
        # default type is 'REG_SZ'
        # Does the state return the correct data
        ret = self.run_state('reg.present',
                             name='HKLM\\{0}'.format(FAKE_KEY),
                             vname='test_reg_sz',
                             vdata='fake string data',
                             use_32bit_registry=True)

        expected = {
            'reg': {
                'Added': {
                    'Entry': 'test_reg_sz',
                    'Key': 'HKLM\\{0}'.format(FAKE_KEY),
                    'Value': 'fake string data'}}}
        self.assertSaltStateChangesEqual(ret, expected)

        # Is the value actually set
        ret = reg.read_value(hive='HKLM',
                             key=FAKE_KEY,
                             vname='test_reg_sz',
                             use_32bit_registry=True)
        expected = {
            'vtype': 'REG_SZ',
            'vname': 'test_reg_sz',
            'success': True,
            'hive': 'HKLM',
            'vdata': 'fake string data',
            'key': FAKE_KEY}
        self.assertEqual(ret, expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Libs
import salt.states.hg as hg

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HgTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Validate the svn state
    '''
    def setup_loader_modules(self):
        return {hg: {}}

    def test_latest(self):
        '''
            Test to Make sure the repository is cloned to
            the given directory and is up to date
        '''
        ret = {'changes': {}, 'comment': '', 'name': 'salt', 'result': True}
        mock = MagicMock(return_value=True)
        with patch.object(hg, '_fail', mock):
            self.assertTrue(hg.latest("salt"))

        mock = MagicMock(side_effect=[False, True, False, False, False, False])
        with patch.object(os.path, 'isdir', mock):
            mock = MagicMock(return_value=True)
            with patch.object(hg, '_handle_existing', mock):
                self.assertTrue(hg.latest("salt", target="c:\\salt"))

            with patch.dict(hg.__opts__, {'test': True}):
                mock = MagicMock(return_value=True)
                with patch.object(hg, '_neutral_test', mock):
                    self.assertTrue(hg.latest("salt", target="c:\\salt"))

            with patch.dict(hg.__opts__, {'test': False}):
                mock = MagicMock(return_value=True)
                with patch.object(hg, '_clone_repo', mock):
                    self.assertDictEqual(hg.latest("salt", target="c:\\salt"),
                                         ret)

    def test_latest_update_changes(self):
        '''
            Test to make sure we don't update even if we have changes
        '''
        ret = {'changes': {}, 'comment': '',
               'name': 'salt', 'result': True}
        revision_mock = MagicMock(return_value='abcdef')
        pull_mock = MagicMock(return_value='Blah.')
        update_mock = MagicMock()

        with patch.dict(hg.__salt__, {'hg.revision': revision_mock,
                                      'hg.pull': pull_mock,
                                      'hg.update': update_mock}):
            mock = MagicMock(side_effect=[True, True])
            with patch.object(os.path, 'isdir', mock):
                mock = MagicMock(return_value=True)
                with patch.dict(hg.__opts__, {'test': False}):
                    with patch.object(hg, '_clone_repo', mock):
                        self.assertDictEqual(hg.latest("salt", target="c:\\salt", update_head=True), ret)
                        assert update_mock.called

    def test_latest_no_update_changes(self):
        '''
            Test to make sure we don't update even if we have changes
        '''
        ret = {'changes': {}, 'comment': 'Update is probably required but update_head=False so we will skip updating.',
               'name': 'salt', 'result': True}
        revision_mock = MagicMock(return_value='abcdef')
        pull_mock = MagicMock(return_value='Blah.')
        update_mock = MagicMock()

        with patch.dict(hg.__salt__, {'hg.revision': revision_mock,
                                      'hg.pull': pull_mock,
                                      'hg.update': update_mock}):
            mock = MagicMock(side_effect=[True, True])
            with patch.object(os.path, 'isdir', mock):
                mock = MagicMock(return_value=True)
                with patch.dict(hg.__opts__, {'test': False}):
                    with patch.object(hg, '_clone_repo', mock):
                        self.assertDictEqual(hg.latest("salt", target="c:\\salt", update_head=False), ret)
                        assert not update_mock.called

    def test_latest_no_update_no_changes(self):
        '''
            Test to Make sure the repository is cloned to
            the given directory and is up to date
        '''
        ret = {'changes': {}, 'comment': 'No changes found and update_head=False so will skip updating.',
               'name': 'salt', 'result': True}
        revision_mock = MagicMock(return_value='abcdef')
        pull_mock = MagicMock(return_value='Blah no changes found.')
        update_mock = MagicMock()

        with patch.dict(hg.__salt__, {'hg.revision': revision_mock,
                                      'hg.pull': pull_mock,
                                      'hg.update': update_mock}):
            mock = MagicMock(side_effect=[True, True])
            with patch.object(os.path, 'isdir', mock):
                mock = MagicMock(return_value=True)
                with patch.dict(hg.__opts__, {'test': False}):
                    with patch.object(hg, '_clone_repo', mock):
                        self.assertDictEqual(hg.latest("salt", target="c:\\salt", update_head=False), ret)
                        assert not update_mock.called
